package com.unis;

import com.trs.idm.client.filter.GeneralSSOFilter;
import com.trs.idm.client.listener.CoSessionListener;
import com.trs.idm.client.servlet.GetLongUrlServlet;
import com.trs.idm.client.servlet.LoginProxyServlet;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.autoconfigure.orm.jpa.HibernateJpaAutoConfiguration;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.boot.web.servlet.ServletListenerRegistrationBean;
import org.springframework.boot.web.servlet.ServletRegistrationBean;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;

@EnableFeignClients(basePackages = "com.unis")
@EnableDiscoveryClient
@ComponentScan("com.unis")
@SpringBootApplication(exclude={DataSourceAutoConfiguration.class, HibernateJpaAutoConfiguration.class})
public class AuthManageApplication {

    public static void main(String[] args) {
        SpringApplication.run(AuthManageApplication.class, args);
    }

    @Bean
    public ServletListenerRegistrationBean<CoSessionListener> servletListenerRegistrationBean() {
        ServletListenerRegistrationBean<CoSessionListener> slrBean = new ServletListenerRegistrationBean<CoSessionListener>();
        slrBean.setListener(new CoSessionListener());
        return slrBean;
    }


    @Bean
    public FilterRegistrationBean servletFilterRegistrationBean() {
        System.err.println("执行过滤器");
        FilterRegistrationBean slrBean = new FilterRegistrationBean();
        slrBean.setFilter(new GeneralSSOFilter()); // 拦截规则
        slrBean.setEnabled(true); // 过滤器顺序
        slrBean.setOrder(2);
        slrBean.addUrlPatterns("/*");
        return slrBean;
    }

    @Bean
    public ServletRegistrationBean servletRegistrationBean() {
        System.err.println("GetLongUrlServlet");
        ServletRegistrationBean sr = new ServletRegistrationBean(new GetLongUrlServlet());
        sr.addUrlMappings("/idsAgents/GetLongUrlServlet");
        sr.setName("GetLongUrlServlet");
        return sr;
    }

    @Bean
    public ServletRegistrationBean servletRegistrationBean2() {
        ServletRegistrationBean sr = new ServletRegistrationBean(new LoginProxyServlet());
        sr.addUrlMappings("/TRSIdSSSOProxyServlet");
        sr.setLoadOnStartup(100);
        sr.setName("TRSIdSSSOProxyServlet");
        return sr;
    }

}
