package com.unis.auth.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.unis.auth.common.QyUserTO;
import com.unis.auth.common.UserDTO;
import com.unis.auth.feign.BaseServerClient;
import com.unis.auth.feign.CompanyClient;
import com.unis.auth.feign.MyUserClient;
import com.unis.auth.utils.NormalSimpleCode;
import com.unis.common.constant.JwtConstant;
import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.EncryptUtil;
import com.unis.common.utils.JwtUtil;
import com.unis.common.utils.ResponseWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

//import com.unis.auth.feign.CompanyClient;

/**
 * @author tiechanglong
 * @version 1.2.0
 * @ClassName: AuthController
 * @Description: TODO
 * @date 2018年12月5日 下午5:24:54
 */
@RestController
@RequestMapping("/auth")
@Api(value = "用户认证", tags = "用户认证")
@Slf4j
public class AuthController {
//    private static final Logger log = LoggerFactory.getLogger(AuthController.class);

    private String effectiveTime = "600000000";

    private String effectiveRefresh = "300000";

    @Autowired
    private MyUserClient myUserClient;

    @Autowired
    private CompanyClient companyClient;

    @Autowired
    private BaseServerClient baseServerClient;

    @PostMapping({"/login/{sign}/{username}/{password}"})
    @ApiOperation(value="用户登陆", notes="sing:用户标识，username:账号，password:密码")
    public ResponseWrapper login(@PathVariable Integer sign, @PathVariable String username, @PathVariable String password, HttpServletRequest request, HttpServletResponse response)
    {
        Map map = new HashMap();
        try
        {
            String userType = sign.toString();
            ResponseWrapper wrapper = this.myUserClient.findByUsername(userType, username);
            Object object = wrapper.getData();
            if ((2 == sign.intValue()) && (ObjectUtils.isEmpty(object))) {
                ResponseWrapper wrapper2 = this.myUserClient.getUserInfoBymobile(username);
                object = wrapper2.getData();
            }
            UserDTO userInfo = (UserDTO)JSON.parseObject(JSON.toJSONString(object), UserDTO.class);
            if (null == userInfo) {
                log.debug(username + "登陆失败，用户不存在");
                return ResponseWrapper.error(Integer.valueOf(4010), "用户不存在", null);
            }
            if (!EncryptUtil.verify(password, userInfo.getUserPassword()))
            {
                log.debug(username + "登陆失败，密码错误");
                return ResponseWrapper.error(Integer.valueOf(4011), "密码错误", null);
            }if (userInfo.getIsDisavled().intValue() == 4011) {
            log.debug(username + "登陆失败，用户被锁定！");
            return ResponseWrapper.error(Integer.valueOf(4012), "用户被锁定！", null);
        }if (wrapper.getInfoCode().intValue() == HttpStatus.OK.value())
        {
            if (2 == sign.intValue()) {
                ResponseWrapper responseWrapper = this.companyClient.checkStatusBySysUserInfoId(userInfo.getUserCode());
                if (responseWrapper.getInfoCode().intValue() != 200) {
                    return ResponseWrapper.error(Integer.valueOf(4013), "用户不存在!", null);
                }
                String string = JSONObject.toJSONString(responseWrapper.getData());
                QyUserTO qyUserTO = (QyUserTO)JSON.parseObject(string, QyUserTO.class);
                if (qyUserTO.getCompanyStatusCode().intValue() != 1) {
                    return ResponseWrapper.success(Integer.valueOf(4016), "当前用户已被管理端停用或注销", null);
                }

                map.put("qyUser", qyUserTO);
            }

            try
            {
                ResponseWrapper qyxx = this.baseServerClient.selectQy(userInfo.getOrgCode());
                if ((qyxx != null) && ("SUCCESS".equalsIgnoreCase(qyxx.getStatus().name())) && (qyxx.getData() != null)) {
                    JSONObject qyxxJson = JSONObject.parseObject(JSONObject.toJSONString(qyxx.getData()));
                    if (qyxxJson != null) {
                        userInfo.setTyshxydm(StringUtils.defaultIfEmpty(qyxxJson.getString("tyshxydm"), ""));
                        userInfo.setCompanyName(StringUtils.defaultIfEmpty(qyxxJson.getString("qymc"), ""));
                    } else {
                        log.error("企业<{}>信息为空", userInfo.getOrgCode());
                    }
                } else {
                    log.error("企业<{}>基础服务获取失败", userInfo.getOrgCode());
                    userInfo.setTyshxydm("");
                    userInfo.setCompanyName("");
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.error(e.getMessage());
            }

            Long effectivTimeLong = Long.valueOf(this.effectiveTime);
            Long effectiveRefreshLong = Long.valueOf(this.effectiveRefresh);
            String token = JwtUtil.createJWT(JwtConstant.JWT_ID, JwtUtil.getClaims(userInfo.getUserCode(), userInfo.getUserLoginName(), userInfo.getUserName(), userInfo
                    .getUserSex(), userInfo.getUserType(), userInfo.getOrgCode(), userInfo.getOrgName(), userInfo.getDeptCode(), userInfo.getDeptName(), userInfo
                    .getCountyCode(), userInfo.getAreaName(), userInfo.getTyshxydm(), userInfo.getCompanyName(), userInfo.getOrgLevel(), userInfo.getSslsxzglbm()), effectivTimeLong
                    .longValue(), effectiveRefreshLong.longValue());
            map.put(JwtConstant.TOKEN, token);
            System.out.println("token:" + token);
            log.debug(username + "登陆成功");
            return ResponseWrapper.success(Integer.valueOf(HttpStatus.OK.value()), HttpStatus.OK.name(), map);
        }
        }
        catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
            return ResponseWrapper.error(Integer.valueOf(HttpStatus.INTERNAL_SERVER_ERROR.value()), HttpStatus.INTERNAL_SERVER_ERROR.name(), null);
        }
        UserDTO userInfo;
        return ResponseWrapper.error(Integer.valueOf(HttpStatus.INTERNAL_SERVER_ERROR.value()), HttpStatus.INTERNAL_SERVER_ERROR.name(), null);
    }

    @PostMapping({"/loginCode/{sign}/{username}/{password}"})
    @ApiOperation(value="用户登陆", notes="sing:用户标识，username:账号，password:密码")
    public void loginCode(@PathVariable Integer sign, @PathVariable String username, @PathVariable String password, @RequestParam("code") String code, HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        Object serverObj = request.getSession().getAttribute("code");
        if ((serverObj != null) && (String.valueOf(serverObj).equalsIgnoreCase(code)))
            request.getRequestDispatcher("/auth/login/" + sign + "/" + username + "/" + password).forward(request, response);
        else
            request.getRequestDispatcher("/auth/verifyFail").forward(request, response);
    }

    @PostMapping({"/checkCode"})
    @ApiOperation(value="检查验证码", notes="sing:用户标识，username:账号，password:密码")
    public ResponseWrapper checkCode(@RequestParam("code") String code, HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        Object serverObj = request.getSession().getAttribute("code");
        if ((serverObj != null) && (String.valueOf(serverObj).equalsIgnoreCase(code))) {
            return ResponseWrapper.success(Integer.valueOf(200), "验证码正确", null);
        }
        request.getRequestDispatcher("/auth/verifyFail").forward(request, response);
        return ResponseWrapper.error(Integer.valueOf(HttpStatus.BAD_REQUEST.value()), "验证码错误", null);
    }
    @PostMapping({"/verifyFail"})
    @ApiIgnore
    public ResponseWrapper message(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException { return ResponseWrapper.error(Integer.valueOf(HttpStatus.BAD_REQUEST.value()), "验证码错误", null); }

    @GetMapping({"/verify"})
    @ApiOperation(value="验证码", notes="验证码")
    public void loginCode(HttpServletRequest request, HttpServletResponse response) throws IOException
    {
        NormalSimpleCode normalSimpleCode = new NormalSimpleCode(100, 30, 4, 5);
        String code = normalSimpleCode.getCode();
        request.getSession().setAttribute("code", code);
        normalSimpleCode.write(response.getOutputStream());
    }
    @ApiOperation(value="获取当前用户信息", notes="参数：无")
    @GetMapping({"/getCurrUserinfo"})
    public ResponseWrapper getCurrUserinfo() { String desc = "获取当前用户信息";
        try {
            UserDTO userDTO = new UserDTO();
            userDTO.setOrgCode(CurrentUtil.getOrgId());
            userDTO.setOrgName(CurrentUtil.getOrgName());
            userDTO.setDeptCode(CurrentUtil.getDeptId());
            userDTO.setDeptName(CurrentUtil.getDeptName());
            userDTO.setUserCode(CurrentUtil.getUserId());
            userDTO.setUserName(CurrentUtil.getUserName());
            userDTO.setUserType(CurrentUtil.getUserType());
            userDTO.setTyshxydm(CurrentUtil.getCreditCode());
            userDTO.setCompanyName(CurrentUtil.getCompanyName());

            return ResponseWrapper.success(Integer.valueOf(200), desc + "成功", userDTO);
        } catch (Exception e) {
            log.error("查找对象，发生异常", e.getMessage());
        }return ResponseWrapper.error(Integer.valueOf(500), "用户信息查询失败", "");
    }

}
