package com.unis.auth.controller;

import com.alibaba.fastjson.JSONObject;
import com.trs.idm.client.actor.ActorException;
import com.trs.idm.client.actor.SSOGroup;
import com.trs.idm.client.actor.SSOUser;
import com.trs.idm.client.actor.StdHttpSessionBasedActor;
import com.unis.auth.common.FindUserPasswdDTO;
import com.unis.auth.common.UserDTO;
import com.unis.auth.feign.BaseServerClient;
import com.unis.auth.feign.MyUserClient;
import com.unis.auth.utils.SpringUtil;
import com.unis.common.constant.JwtConstant;
import com.unis.common.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class IdentityActor extends StdHttpSessionBasedActor {


    /**
     * 同步ids的用户信息到应用
     */
    @Override
    public boolean addUser(SSOUser user, HttpServletRequest request) throws ActorException {
        log.info("------addUser------user:{}", user);
        return addUpdateUser(user);
    }

    public boolean checkLocalLogin(HttpSession session) {
        return false;
    }

    /**
     * 检查登录，判断协作应用当前session是否登录
     *
     * @return
     */
    public boolean checkLocalLogin(HttpServletRequest request, HttpSession session) {
        String token = request.getHeader(JwtConstant.TOKEN);
        if (token == null) {
            Object obj = session.getAttribute(JwtConstant.TOKEN);
            if (obj != null) {
                token = String.valueOf(obj);
            }
        }
        Object urlObj = session.getAttribute("url");
        if (urlObj != null) {
            session.setAttribute("url", urlObj);
        }
        boolean flag = false;
        if (token != null) {
            try {
                String userAccount = CurrentUtil.getUserAccount(token);
                flag = userAccount != null;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        log.info("------checkLocalLogin------登录状态:{}", flag + "   sessionId:" + session.getId());
        return flag;
    }

    @Override
    public String extractUserName(HttpServletRequest request) throws ActorException {
        String url = request.getParameter("url");
        if (url != null) {
            request.getSession().setAttribute("url", url);
        }
        String type = request.getParameter("type");
        if (type != null) {
            request.getSession().setAttribute("type", type);
        }

        String username = null;
        if (request != null) {
            username = request.getParameter("username");
        }
        log.info("------extractUserName-----username:{}", username);
        return username;
    }

    @Override
    public String extractUserPwd(HttpServletRequest request) throws ActorException {
        String password = null;
        if (request != null) {
            password = request.getParameter("password");
        }
        log.info("------extractUserPwd------password:{}", password);
        return password;
    }

    @Override
    public void loadAnonymous(HttpServletRequest request) throws ActorException {
        log.info("------loadAnonymous------request:{}", request);
    }

    public void loadLoginUser(HttpServletRequest request, SSOUser user) throws ActorException {

    }

    /**
     * 执行登录： 加载登录的统一用户到协作应用的当前会话（session对象）中，完成协作应用自己的登录逻辑（不需要再次对用户进行认证，只需要加载）
     */
    public void loadLoginUserNew(HttpServletRequest request, HttpServletResponse response, SSOUser user) throws ActorException {
        log.info("------loadLoginUser------user:{}", user.getUserName());
        BaseServerClient baseServerClient = SpringUtil.getBean(BaseServerClient.class);
        MyUserClient userServer = SpringUtil.getBean(MyUserClient.class);
        /*String userStr= ObjectUtils.toString(request.getSession().getAttribute("type"),null);
        int userType=1;
        if(userStr!=null&&userStr.matches("\\d")){
            userType=Integer.valueOf(userStr);
        }*/
        try {
            ResponseWrapper responseWrapper = userServer.findByUsername(user.getUserName());
            if (responseWrapper.isSuccess() && responseWrapper.getData() != null) {
                UserDTO userDTO = JsonUtils.parse(JsonUtils.serialize(responseWrapper.getData()), UserDTO.class);
                if (userDTO != null && userDTO.getOrgCode() != null) {
                    try {
                        ResponseWrapper qyxx = baseServerClient.selectQy(userDTO.getOrgCode());
                        if (qyxx != null && "SUCCESS".equalsIgnoreCase(qyxx.getStatus().name()) && qyxx.getData() != null) {
                            JSONObject qyxxJson = JSONObject.parseObject(JsonUtils.serialize(qyxx.getData()));
                            String tyshxydm = qyxxJson.getString("tyshxydm");
                            userDTO.setTyshxydm(tyshxydm);
                            log.info("获取统一社会信用代码成功:" + JsonUtils.serialize(qyxxJson));
                    /*	//获取kddm
                        JSONObject param = new JSONObject();
                        param.put("tyshxydm", tyshxydm);
                        ResponseWrapper kdxx = baseServerClient.getDKdxxsjyList(param);
                        if (kdxx != null && "SUCCESS".equalsIgnoreCase(kdxx.getStatus().name())  && kdxx.getData() != null) {
                            JSONArray kdxxJsonArray = JSONArray.parseArray(JsonUtils.serialize(kdxx.getData()));
                            if (kdxxJsonArray.size() > 0) {
                                String kddm = kdxxJsonArray.getJSONObject(0).getString("kddm");
                                userDTO.setKddm(kddm);
                            }
                        }*/
                        } else {
                            userDTO.setTyshxydm("");
                            ///userDTO.setKddm("");
                            log.error("获取统一社会信用代码失败:" + JsonUtils.serialize(qyxx));
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    long effectiveTime = 600000000;

                    long effectiveRefresh = 300000;
                    String token = JwtUtil.createJWT(JwtConstant.JWT_ID, JwtUtil.getClaims(userDTO.getUserCode(), userDTO.getUserLoginName(), userDTO.getUserName(),
                            userDTO.getUserSex(), userDTO.getUserType(), userDTO.getOrgCode(), userDTO.getOrgName(), userDTO.getDeptCode(), userDTO.getDeptName(),
                            userDTO.getCountyCode(), userDTO.getUserAddress(), userDTO.getTyshxydm(), userDTO.getCompanyName(), "", ""), effectiveTime, effectiveRefresh);
                    request.setAttribute(JwtConstant.TOKEN, token);

                    //添加cookie,以兼容单点登录
                  /*  Cookie cookie = new Cookie(JwtConstant.TOKEN, token);
                    cookie.setPath("/");
                    response.addCookie(cookie);*/

                    //也添加上session,但基本用不上,只用于ids退出通知应用可以退出
                    request.getSession().setAttribute(JwtConstant.TOKEN, token);
                    System.out.println("生成的token:" + token);
                } else {
                    log.error("用户信息为空:" + user.getUserName());
                    System.out.println(JsonUtils.serialize(responseWrapper));
                }
            } else {
                log.error("myuser服务返回信息为空:" + user.getUserName());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 注销退出： 完成协作应用自己的退出逻辑
     */
    public void logout(HttpSession session) {
        log.info("------logout------");
        Object tokenObj = session.getAttribute(JwtConstant.TOKEN);
        if (tokenObj != null) {
            session.removeAttribute(JwtConstant.TOKEN);
            //removeCookieToken();
            session.invalidate();
        }
    }

    /**
     * 同步ids的用户信息到应用
     */
    @Override
    public boolean removeUser(SSOUser user, HttpServletRequest request) throws ActorException {
        log.info("------removeUser------user:{}", user);
        return false;
    }

    /**
     * 同步ids的用户信息到应用
     */
    @Override
    public boolean updateUser(SSOUser user, HttpServletRequest request) throws ActorException {
        log.info("------updateUser------user:{}", user);
        return addUpdateUser(user);
    }

    public boolean disableUser(SSOUser user) throws ActorException {
        log.info("------disableUser------user:{}", user);
        MyUserClient userServer = SpringUtil.getBean(MyUserClient.class);
        boolean bl = false;
        try {
            ResponseWrapper responseWrapper = userServer.findByUsername(user.getUserName());
            if (responseWrapper.isSuccess()) {
                UserDTO userDb = JsonUtils.parse(JsonUtils.serialize(responseWrapper.getData()), UserDTO.class);
                if (userDb != null) {
                    bl = userServer.freezeUser(userDb.getUserCode()).isSuccess();
                }
            }
        } catch (Exception e) {
            log.error("ids禁用用户失败loginName:{}  userName:{}", user.getUserName(), user.getTrueName(), e);
        }
        return bl;
    }

    public boolean enableUser(SSOUser user) throws ActorException {
        log.info("------enableUser------user:{}", user);
        MyUserClient userServer = SpringUtil.getBean(MyUserClient.class);
        boolean bl = false;
        try {
            ResponseWrapper responseWrapper = userServer.findByUsername(user.getUserName());
            if (responseWrapper.isSuccess()) {
                UserDTO userDb = JsonUtils.parse(JsonUtils.serialize(responseWrapper.getData()), UserDTO.class);
                if (userDb != null) {
                    bl = userServer.freezeUser(userDb.getUserCode()).isSuccess();
                }
            }
        } catch (Exception e) {
            log.error("ids启用用户失败loginName:{}  userName:{}", user.getUserName(), user.getTrueName(), e);
        }
        return bl;
    }

    @Override
    public boolean userExist(SSOUser user) throws ActorException {
        log.info("------userExist------user:{}", user);
        MyUserClient userServer = SpringUtil.getBean(MyUserClient.class);
        boolean success = false;
        try {
            ResponseWrapper responseWrapper = userServer.findByUsername(user.getUserName());
            if (responseWrapper.isSuccess()) {
                success = true;
            }
        } catch (Exception e) {
            log.error("ids请求判断用户存在失败", e);
        }
        return success;
    }

    public boolean addGroup(SSOGroup group, HttpServletRequest req) {
        log.info("------addGroup------groups:{}", group);
        return true;
    }

    public boolean delGroup(SSOGroup group, HttpServletRequest req) {
        log.info("------delGroup------groups:{}", group);
        return true;
    }

    public boolean updateGroup(SSOGroup group, HttpServletRequest req) {
        log.info("------updateGroup------groups:{}", group);
        return true;
    }

    public boolean moveToGroup(SSOUser user, SSOGroup group) throws ActorException {
        log.info("------moveToGroup:{}------groups:{}", user, group);
        return true;
    }

    public boolean removeFromGroup(SSOUser user, SSOGroup group) throws ActorException {
        log.info("------removeFromGroup:{}------ from groups:{}", user, group);
        return true;
    }


    private boolean addUpdateUser(SSOUser user) {
        synchronized (user.getUserName().intern()) {
            MyUserClient userServer = SpringUtil.getBean(MyUserClient.class);
            UserDTO userDTO = new UserDTO();
            userDTO.setUserLoginName(user.getUserName());
            if (StringUtils.isNotEmpty(user.getTrueName())) {
                userDTO.setUserName(user.getTrueName());
            } else {
                String firstName = StringUtils.defaultIfEmpty(user.getProperty("firstName"), "");
                String lastName = StringUtils.defaultIfEmpty(user.getProperty("lastName"), "");
                String userName = StringUtils.defaultIfEmpty(firstName + lastName, user.getUserName());
                userDTO.setUserName(userName);
            }
            userDTO.setUserMobile(user.getProperty("mobile"));
            //	用户等级 存储用户类型
            userDTO.setUserType(user.getProperty("level"));
            userDTO.setUserSex(user.getProperty("gender"));
            userDTO.setUserImgurl(user.getProperty("headUrl"));
            //职务
            userDTO.setUserTitle(user.getProperty("position"));
            userDTO.setUserCertno(user.getProperty("creditID"));
            userDTO.setUserAddress(user.getProperty("address"));
            userDTO.setUserPost(user.getProperty("postCode"));
            userDTO.setUserTel(user.getProperty("tel"));
            userDTO.setUserMail(user.getProperty("email"));
            userDTO.setIsDisavled(0);
            String defUserType = "1";
            String plainUserPwd = user.getProperty("plainUserPwd");
            userDTO.setUserPassword(plainUserPwd);
            try {
                ResponseWrapper responseWrapper = userServer.findByUsername(user.getUserName());
                UserDTO userDb = null;
                if (responseWrapper.isSuccess() && responseWrapper.getData() != null) {
                    userDb = JsonUtils.parse(JsonUtils.serialize(responseWrapper.getData()), UserDTO.class);
                }
                if (userDb == null) {
                    userDTO.setCountyCode("120000");
                    //内置机构编码
                    userDTO.setOrgCode("LS001");
                    //内置部门编码
                    userDTO.setDeptCode("DP001");
                    //密码
                    userDTO.setUserPassword(plainUserPwd);

                    String userType = user.getProperty("userType");
                    userDTO.setUserType(StringUtils.defaultIfEmpty(userType, defUserType));
                    userDTO.setSpeciality("新增-" + Thread.currentThread().getName() + "-"
                            + DateUtil.getDateTime());
                    ResponseWrapper resp = userServer.saveUserinfo("new", userDTO);
                    if (resp.isSuccess()) {
                        String userCode = String.valueOf(resp.getData());
                        List<String> list = new ArrayList<>();
                        //内置角色编码
                        list.add("R0000001");
                        userDTO.setUserRole(list);
                        userServer.saveUserRoles(list, userCode);
                    }
                } else {
                    userDTO.setUserCode(userDb.getUserCode());
                    userDTO.setSpeciality("更新-----------" + DateUtil.getDateTime());
                    userServer.saveUserinfo("edit", userDTO);
                    if (plainUserPwd != null && userDb.getUserPassword() != null && !userDb.getUserPassword()
                            .equals(EncryptUtil.generate(plainUserPwd))) {
                        log.info("用户<{},{}>密码<{}>同步", userDb.getUserCode(), userDb.getUserLoginName()
                                , userDb.getUserPassword());
                        FindUserPasswdDTO findUserPasswdDTO = new FindUserPasswdDTO();
                        findUserPasswdDTO.setUserCode(userDb.getUserCode());
                        findUserPasswdDTO.setUserLoginName(userDb.getUserLoginName());
                        findUserPasswdDTO.setNewPasswd1(plainUserPwd);
                        findUserPasswdDTO.setNewPasswd2(plainUserPwd);
                        userServer.findPwd(findUserPasswdDTO);
                    } else {
                        log.info("用户<{}>密码<{}>无需修改", user.getUserName(), userDb.getUserPassword());
                    }
                }
                return true;
            } catch (Exception e) {
                log.error("ids同步或更新用户失败loginName:{}  userName:{}", user.getUserName(), user.getTrueName(), e);
            }
            return false;
        }
    }

    /**
     * 删除cookie中的token信息
     */
    private void removeCookieToken() {
        Cookie[] cookies = RequestUtils.getRequest().getCookies();
        if (null != cookies && cookies.length > 0) {
            for (Cookie cookie : cookies) {
                if (cookie.getName().equals(JwtConstant.TOKEN)) {
                    //if ( cookie.getName().equals("TOKENL") ||cookie.getName().equals("TOKENB") )
                    CookieUtil.removeCookie(cookie.getName());
                }
            }
        }
    }
}
