package com.unis.auth.feign;

import com.unis.auth.common.FindUserPasswdDTO;
import com.unis.auth.common.UserDTO;
import com.unis.common.utils.ResponseWrapper;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 
 * @ClassName: MyUserClient 
 * @Description:  
 * @author tiechanglong
 * @date 2018年12月5日 下午5:11:16 
 * 
 * @version 1.2.0
 */
@FeignClient(name = "myuser")
@Component
public interface MyUserClient {

	/**
	 * 
	 * @Description: 登陆接口
	 *
	 * @date 2018年12月5日,下午5:20:42
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @param sign
	 * @param username
	 * @param password
	 * @return
	 */
    @RequestMapping(method = RequestMethod.POST,value = "/user/login/{sign}/{username}/{password}")
    ResponseWrapper login(@PathVariable("sign") Integer sign, @PathVariable("username") String username, @PathVariable("password") String password);

	/**
	 * @return ResponseWrapper
	 * @Date: 时间:2018/5/25 15:10
	 * * @param name
	 */
	@ApiOperation(value = "根据登录名获取用户信息", notes = "参数：用户账号")
	@GetMapping(value = "/user/findByUserloginname/{userLoginName}")
	public ResponseWrapper findByUsername(@PathVariable("userLoginName") String userLoginName);

	@ApiOperation(value = "添加或修改用户", notes = "参数：type（string，必填）：类型，new 新增 / edit 修改")
	@PostMapping(value = "/user/saveUserinfo/{type}")
	public ResponseWrapper saveUserinfo(@PathVariable("type") String type, @RequestBody UserDTO userDTO);


	/**
	 * 冻结/解冻该账户
	 *
	 * @param
	 * @return
	 */
	@ApiOperation(value = "冻结用户", notes = "参数：用户id")
	@PostMapping(value = "/user/freezeUser")
	public ResponseWrapper freezeUser(String userCode);


	@ApiOperation(value = "保存或更新用户角色信息", notes = "参数：roleIdsVO")
	@PostMapping(value = "/user/saveUserRoles")
	public ResponseWrapper saveUserRoles(@RequestBody List<String> roleCodes, @RequestParam("userCode") String userCode);

	/**
	 *
	 * @Description: 登陆接口
	 *
	 * @date 2018年12月5日,下午5:20:42
	 * @author tiechanglong
	 * @version 1.2.0
	 * @return
	 */
	@RequestMapping(method = RequestMethod.GET,value = "/user/findByUsername/{userType}/{userLoginName}")
	ResponseWrapper findByUsername(@RequestParam("userType") String userType, @RequestParam("userLoginName") String userLoginName);


	/**
	 * @param mobile
	 * @return根据手机号获取用户信息
	 */
	@RequestMapping(method = RequestMethod.GET,value = "/user/getUserInfoBymobile/{mobile}")
	ResponseWrapper getUserInfoBymobile(@RequestParam("mobile") String mobile);

	@ApiOperation(value = "找回密码", notes = "账号、新密码")
	@PostMapping(value = "/user/findPwd")
	public ResponseWrapper findPwd(@RequestBody FindUserPasswdDTO dto);
}
