package com.unis.auth.utils;

import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

/**
 * @author qingsong.han
 * @description:
 * @create 2024-03-28 14:41
 */
@Slf4j
public class Base64Reverse {

    private static final char[] BASE64CODE = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '_', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' };

    private static final byte[] BASE64DECODE = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -1, -1, 0, -1, -1, -1, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, -1, -1, -1, -1, 37, -1, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, -1, -1, -1, -1, -1 };
    private static final int HEX_255 = 255;
    private static final int HEX_16515072 = 16515072;
    private static final int HEX_258048 = 258048;
    private static final int HEX_4032 = 4032;
    private static final int HEX_63 = 63;
    private static final int HEX_16711680 = 16711680;
    private static final int HEX_65280 = 65280;
    private static final int NUMBER_TWO = 2;
    private static final int NUMBER_THREE = 3;
    private static final int NUMBER_FOUR = 4;
    private static final int NUMBER_SIX = 6;
    private static final int NUMBER_EIGHT = 8;
    private static final int NUMBER_TWELVE = 12;
    private static final int NUMBER_SIXTEEN = 16;
    private static final int NUMBER_EIGHTEEN = 18;

    public static String encode(byte[] b)
    {
        int code = 0;

        StringBuffer sb = new StringBuffer((b.length - 1) / 3 << 6);

        for (int i = 0; i < b.length; i++) {
            code |= b[i] << 16 - i % 3 * 8 & 255 << 16 - i % 3 * 8;

            if ((i % 3 == 2) || (i == b.length - 1)) {
                sb.append(BASE64CODE[((code & 0xFC0000) >>> 18)]);
                sb.append(BASE64CODE[((code & 0x3F000) >>> 12)]);
                sb.append(BASE64CODE[((code & 0xFC0) >>> 6)]);
                sb.append(BASE64CODE[(code & 0x3F)]);
                code = 0;
            }

        }

        if (b.length % 3 > 0) {
            sb.setCharAt(sb.length() - 1, '=');
        }
        if (b.length % 3 == 1) {
            sb.setCharAt(sb.length() - 2, '=');
        }
        return sb.toString();
    }

    public static byte[] decode(String code)
    {
        if (code == null) {
            return null;
        }
        int len = code.length();
        if (len % 4 != 0) {
            throw new IllegalArgumentException("Base64 string length must be 4*n");
        }
        if (code.length() == 0) {
            return new byte[0];
        }

        int pad = 0;
        if (code.charAt(len - 1) == '=') {
            pad++;
        }
        if (code.charAt(len - 2) == '=') {
            pad++;
        }

        int retLen = len / 4 * 3 - pad;

        byte[] ret = new byte[retLen];

        for (int i = 0; i < len; i += 4) {
            int j = i / 4 * 3;
            char ch1 = code.charAt(i);
            char ch2 = code.charAt(i + 1);
            char ch3 = code.charAt(i + 2);
            char ch4 = code.charAt(i + 3);
            int tmp = BASE64DECODE[ch1] << 18 | BASE64DECODE[ch2] << 12 | BASE64DECODE[ch3] << 6 | BASE64DECODE[ch4];

            ret[j] = (byte)((tmp & 0xFF0000) >> 16);
            if (i < len - 4) {
                ret[(j + 1)] = (byte)((tmp & 0xFF00) >> 8);
                ret[(j + 2)] = (byte)(tmp & 0xFF);
            }
            else {
                if (j + 1 < retLen) {
                    ret[(j + 1)] = (byte)((tmp & 0xFF00) >> 8);
                }
                if (j + 2 < retLen) {
                    ret[(j + 2)] = (byte)(tmp & 0xFF);
                }
            }
        }
        return ret;
    }

    public static String strToBase64(String str) {
        if (str != null) {
            byte[] a = new byte[0];
            try {
                a = str.getBytes("UTF-8");
            } catch (UnsupportedEncodingException e) {
                log.error("不支持的编码");
            }
            int aLen = a.length;
            int numFullGroups = aLen / 3;
            int numBytesInPartialGroup = aLen - 3 * numFullGroups;
            int resultLen = 4 * ((aLen + 2) / 3);
            StringBuffer result = new StringBuffer(resultLen);

            int inCursor = 0;
            for (int i = 0; i < numFullGroups; i++) {
                int byte0 = a[(inCursor++)] & 0xFF;
                int byte1 = a[(inCursor++)] & 0xFF;
                int byte2 = a[(inCursor++)] & 0xFF;
                result.append(BASE64CODE[(byte0 >> 2)]);
                result.append(BASE64CODE[(byte0 << 4 & 0x3F | byte1 >> 4)]);
                result.append(BASE64CODE[(byte1 << 2 & 0x3F | byte2 >> 6)]);
                result.append(BASE64CODE[(byte2 & 0x3F)]);
            }

            if (numBytesInPartialGroup != 0) {
                int byte0 = a[(inCursor++)] & 0xFF;
                result.append(BASE64CODE[(byte0 >> 2)]);

                if (numBytesInPartialGroup == 1) {
                    result.append(BASE64CODE[(byte0 << 4 & 0x3F)]);
                    result.append("==");
                }
                else {
                    int byte1 = a[(inCursor++)] & 0xFF;
                    result.append(BASE64CODE[(byte0 << 4 & 0x3F | byte1 >> 4)]);
                    result.append(BASE64CODE[(byte1 << 2 & 0x3F)]);
                    result.append('=');
                }
            }
            return result.toString();
        }
        return null;
    }

    public static String base64ToStr(String s) {
        if (s != null)
        {
            int sLen = s.length();
            int numGroups = sLen / 4;
            if (4 * numGroups != sLen) {
                throw new IllegalArgumentException("亲,是不是忘了加密了:" + s);
            }

            int missingBytesInLastGroup = 0;
            int numFullGroups = numGroups;
            if (sLen != 0)
            {
                if (s.charAt(sLen - 1) == '=') {
                    missingBytesInLastGroup++;

                    numFullGroups--;
                }

                if (s.charAt(sLen - 2) == '=') {
                    missingBytesInLastGroup++;
                }
            }
            byte[] result = new byte[3 * numGroups - missingBytesInLastGroup];
            int inCursor = 0; int outCursor = 0;
            for (int i = 0; i < numFullGroups; i++) {
                int ch0 = base64toInt(s.charAt(inCursor++), BASE64DECODE);
                int ch1 = base64toInt(s.charAt(inCursor++), BASE64DECODE);
                int ch2 = base64toInt(s.charAt(inCursor++), BASE64DECODE);
                int ch3 = base64toInt(s.charAt(inCursor++), BASE64DECODE);
                result[(outCursor++)] = (byte)(ch0 << 2 | ch1 >> 4);
                result[(outCursor++)] = (byte)(ch1 << 4 | ch2 >> 2);
                result[(outCursor++)] = (byte)(ch2 << 6 | ch3);
            }
            if (missingBytesInLastGroup != 0) {
                int ch0 = base64toInt(s.charAt(inCursor++), BASE64DECODE);
                int ch1 = base64toInt(s.charAt(inCursor++), BASE64DECODE);

                result[(outCursor++)] = (byte)(ch0 << 2 | ch1 >> 4);

                if (missingBytesInLastGroup == 1) {
                    int ch2 = base64toInt(s.charAt(inCursor++), BASE64DECODE);
                    result[(outCursor++)] = (byte)(ch1 << 4 | ch2 >> 2);
                }
            }
            try {
                return new String(result, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                log.error("不支持的编码");
            }
        }
        return null;
    }

    private static int base64toInt(char c, byte[] alphaToInt) {
        int result = alphaToInt[c];
        if (result < 0) {
            log.error(c + ":不是加密因子,无法解密");
            throw new IllegalArgumentException("非法参数");
        }
        return result;
    }

    public static void main(String[] args) throws IOException {
        System.out.println("61   =");
        System.out.println("45   -");
        System.out.println("95   _");
        System.out.println("*   42");
        System.out.println("+   43");
        System.out.println(",   44");
        System.out.println("-   45");
        System.out.println(".   46");
        System.out.println("/   47");
        System.out.println("0   48");
        String[] strArr = { "飞机飞起来", "abc", "测试", "好友", "同学们", "如何行走", "行歧路", "123*测", "123,测", "123.测", "-_", "*", "+///+++" };

        String base64 = null;

        System.out.println(base64ToStr("tM9InlP_8yCxbi4Kbh8G8B1wrRcRqR9z8yCxbi8Nc3jQcNvT8xKxsAkMploDnQ5OplXIfQXzoh8U8ADObNbxuf=="));
        System.out.println(base64ToStr("bi4Kbf=="));
        System.out.println(base64ToStr("bi8Nc3jQcNvT"));
        System.out.println(base64ToStr("cRoLcv=="));
    }

}
